/****************************************************************************
*                mechsim.h
*
*  Author: Christoph Hormann <chris_hormann@gmx.de>
*
*  Mechanics simulation
*
*  for Persistence of Vision(tm) Ray Tracer
*  Copyright 2002 Christoph Hormann
*
*  MechSim version 0.1.0   October 28, 2002
*
*****************************************************************************/


#ifndef MECHSIM_H
#define MECHSIM_H

/*****************************************************************************
* Global preprocessor defines
******************************************************************************/

#define MECHSIM_METHOD_EULER 1
#define MECHSIM_METHOD_HEUN 2
#define MECHSIM_METHOD_GRADIENT 4

#define MECHSIM_FLAG_FIXED 1

#define MECHSIM_COLLISION_NONE 0
#define MECHSIM_COLLISION_ALL 1
#define MECHSIM_COLLISION_GROUP 2

#define MECHSIM_ENV_METHOD_FORCE 1
#define MECHSIM_ENV_METHOD_IMPACT 2

/*****************************************************************************
* Global typedefs
******************************************************************************/

/* ------------------------------------------------------ */
typedef struct mechsim_state_struct MECHSIM_STATE;

struct mechsim_state_struct {
  VECTOR Position;
  VECTOR Velocity;
};

/* ------------------------------------------------------ */
typedef struct mechsim_mass_struct MECHSIM_MASS;

struct mechsim_mass_struct {

  int Index;

  VECTOR Position;
  VECTOR Velocity;
  DBL Mass;
  DBL Radius;
  int Flag;

  int Group;
};

/* ------------------------------------------------------ */
typedef struct mechsim_connect_struct MECHSIM_CONNECT;

struct mechsim_connect_struct {

  int Index;

  int Idx1;
  int Idx2;

  DBL Length;
  DBL Stiffness;
  DBL Damping;

  int Group;
};

/* ------------------------------------------------------ */
typedef struct mechsim_face_struct MECHSIM_FACE;

struct mechsim_face_struct {

  int Index;

  int Idx1;
  int Idx2;
  int Idx3;

  int Group;
};

/* ------------------------------------------------------ */
typedef struct mechsim_group_struct MECHSIM_GROUP;

struct mechsim_group_struct {

  int max_masses;
  int max_connects;
  int max_faces;

  int mass_count;
  int connect_count;
  int faces_count;

  MECHSIM_MASS **Masses;
  MECHSIM_CONNECT **Connects;
  MECHSIM_FACE **Faces;
};

/* ------------------------------------------------------ */
typedef struct mechsim_data_struct MECHSIM_DATA;

struct mechsim_data_struct {

  int max_masses;
  int max_connects;
  int max_faces;
  int max_groups;

  int mass_count;
  int connect_count;
  int faces_count;
  int group_count;

  MECHSIM_MASS *Masses;
  MECHSIM_CONNECT *Connects;
  MECHSIM_FACE *Faces;
  MECHSIM_GROUP *Groups;

  OBJECT *Faces_Object;
};

/* ------------------------------------------------------ */
typedef struct mechsim_environment_struct MECHSIM_ENVIRONMENT;

struct mechsim_environment_struct {

  FUNCTION_PTR Function;
  OBJECT *Object;
  DBL Stiffness;
  DBL Damping;
  DBL Friction;
  DBL Friction_Excess;
  unsigned int Method;

  MECHSIM_ENVIRONMENT *Next;
};

/* ------------------------------------------------------ */
typedef struct mechsim_options_struct MECHSIM_OPTIONS;

struct mechsim_options_struct {

  unsigned int Method;
  int Step_Count;
  DBL Time_Step;

  VECTOR gravity;
  DBL Damping;

  MECHSIM_ENVIRONMENT *Environment;

  DBL Collision_Stiffness;
  DBL Collision_Damping;
  DBL Collision_Friction;
  DBL Collision_Friction_Excess;
  int Calc_Collision;
  int Calc_Collision_Faces;

  char* Load_File_Name;
  char* Save_File_Name;

  bool Save_File;

  bool Enabled;

  MECHSIM_DATA Data;

};

/* ------------------------------------------------------ */
/* global functions */
/* ------------------------------------------------------ */

bool Mechsim_read_file (POV_ISTREAM * fd, int Group);

void Initialize_MechSim(void);
void Deinitialize_MechSim(void);

void Mechsim_Simulate(void);



extern MECHSIM_OPTIONS MechsimOptions;

#endif

